@@ -23,6 +23,7 @@ class TransformerBlock(Layer):
         ff_dim: int,
         att_dropout: float = 0.1,
         ff_dropout: float = 0.1,
+        explainable = False
     ):
         """Transformer model for TabTransformer
 
@@ -32,8 +33,10 @@ class TransformerBlock(Layer):
             ff_dim (int): size of feed-forward layer
             att_dropout (float, optional): dropout rate in multi-headed attention layer. Defaults to 0.1.
             ff_dropout (float, optional): dropout rate in feed-forward layer. Defaults to 0.1.
+            explainable (bool, optional): if True, returns attention weights
         """
         super(TransformerBlock, self).__init__()
+        self.explainable = explainable
         self.att = MultiHeadAttention(
             num_heads=num_heads, key_dim=embed_dim, dropout=att_dropout
         )
@@ -46,8 +49,13 @@ class TransformerBlock(Layer):
         self.skip2 = Add()
 
     def call(self, inputs):
-        # Multi headed attention
-        attention_output = self.att(inputs, inputs)
+        if self.explainable:
+            # Multi headed attention with attentio nscores
+            attention_output, att_weights = self.att(inputs, inputs, return_attention_scores=True)
+        else:
+            # Without attention
+            attention_output = self.att(inputs, inputs)
+
         # Skip connection
         attention_output = self.skip1([inputs, attention_output])
         # Layer norm
@@ -58,8 +66,11 @@ class TransformerBlock(Layer):
         feedforward_output = self.skip2([feedforward_output, attention_output])
         # Layer norm
         transformer_output = self.layernorm2(feedforward_output)
-
-        return transformer_output
+        
+        if self.explainable:
+            return transformer_output, att_weights
+        else:
+            return transformer_output
 
 
 class TabTransformerEncoder(tf.keras.Model):
@@ -73,7 +84,7 @@ class TabTransformerEncoder(tf.keras.Model):
         heads: int = 8,
         attn_dropout: float = 0.1,
         ff_dropout: float = 0.1,
-        numerical_discretisers: dict = None,
+        numerical_embeddings: dict = None,
         use_column_embedding: bool = True,
     ):
         """TabTransformer Tensorflow Model
@@ -96,7 +107,7 @@ class TabTransformerEncoder(tf.keras.Model):
         super(TabTransformerEncoder, self).__init__()
         self.numerical = numerical_features
         self.categorical = categorical_features
-        self.quantize = numerical_discretisers is not None
+        self.embed_numeric = numerical_embeddings is not None
         self.num_categories = [
             categorical_lookup[c].vocabulary_size() for c in self.categorical
         ]
@@ -104,26 +115,14 @@ class TabTransformerEncoder(tf.keras.Model):
         # ---------- Numerical Input -----------
         if len(self.numerical) > 0:
             # If we want to quantise numeric features
-            if self.quantize:
-                self.num_bins = [
-                    numerical_discretisers[n][1].vocabulary_size()
-                    for n in self.numerical
+            if self.embed_numeric:
+                # Layers to transform numeric into embedding
+                self.numerical_embeddings = numerical_embeddings
+                # Linear layer after embedding
+                self.numerical_embedding_linear = [
+                    Dense(embedding_dim, activation='relu') for n in self.numerical
                 ]
-                # Discretisation layers
-                self.numerical_discretisers = [
-                    numerical_discretisers[n][0] for n in self.numerical
-                ]
-                # Lookup layers
-                self.numerical_lookup = [
-                    numerical_discretisers[n][1] for n in self.numerical
-                ]
-                # Embedding layers
-                self.num_embedding_layers = []
-                for bins in self.num_bins:
-                    numerical_embedding = Embedding(
-                        input_dim=bins, output_dim=embedding_dim
-                    )
-                    self.num_embedding_layers.append(numerical_embedding)
+                
             else:
                 # If not quantising, then simply normalise and concatenate
                 self.continuous_normalization = LayerNormalization()
@@ -146,7 +145,7 @@ class TabTransformerEncoder(tf.keras.Model):
         self.use_column_embedding = use_column_embedding
         if use_column_embedding:
             num_columns = len(self.categorical)
-            if self.quantize:
+            if self.embed_numeric:
                 num_columns += len(self.numerical)
             self.column_embedding = Embedding(
                 input_dim=num_columns, output_dim=embedding_dim
@@ -180,10 +179,9 @@ class TabTransformerEncoder(tf.keras.Model):
         if len(self.numerical) > 0:
             # Each numeric feature needs to be binned, looked up, and embedded
             for i, n in enumerate(self.numerical):
-                if self.quantize:
-                    num_binned = self.numerical_discretisers[i](inputs[n])
-                    num_binned = self.numerical_lookup[i](num_binned)
-                    num_embedded = self.num_embedding_layers[i](num_binned)
+                if self.embed_numeric:
+                    num_embedded = self.numerical_embeddings[n](inputs[n])
+                    num_embedded = self.numerical_embedding_linear[i](num_embedded)
                     numerical_feature_list.append(num_embedded)
                 else:
                     # Otherwise we pass it as it is
@@ -194,7 +192,7 @@ class TabTransformerEncoder(tf.keras.Model):
             cat_embedded = self.cat_embedding_layers[i](cat_encoded)
             categorical_feature_list.append(cat_embedded)
 
-        if self.quantize:
+        if self.embed_numeric:
             # Stack categorical embeddings for the Tansformer.
             transformer_inputs = self.embedded_concatenation(
                 numerical_feature_list + categorical_feature_list
@@ -213,14 +211,13 @@ class TabTransformerEncoder(tf.keras.Model):
         mlp_input = self.flatten_transformer_output(transformer_inputs)
 
         # In case we don't quantize, we want to normalise and concatenate numerical features with embeddings
-        if (self.quantize is False) and (len(self.numerical) > 0):
+        if (self.embed_numeric is False) and (len(self.numerical) > 0):
             numerical_inputs = self.numerical_concatenation(numerical_feature_list)
             numerical_inputs = self.continuous_normalization(numerical_inputs)
             mlp_input = self.pre_mlp_concatenation([mlp_input, numerical_inputs])
 
         return mlp_input
 
-
 class TabTransformerRTD(tf.keras.Model):
     def __init__(
         self,
@@ -316,7 +313,7 @@ class TabTransformer(tf.keras.Model):
             )
 
         # mlp layers
-        if self.encoder.quantize:
+        if self.encoder.embed_numeric:
             mlp_input_dim = embedding_dim * (
                 len(self.encoder.numerical) + len(self.encoder.categorical)
             )
